from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.candellation_details_enums import PartyEnum, ReasonEnum


class CancellationDetails(BaseModel):
    party: PartyEnum
    reason: ReasonEnum

    model_config = ConfigDict(use_enum_values=True)



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.deal_submodels.settlements_model import (
    Settlement,
)


class Deal(BaseModel):
    id: str = Field(min_length=36, max_length=50)
    settlements: list[Settlement]



from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.delivery_method_enums import DeliveryMethodTypeEnum


class DeliveryMethod(BaseModel):
    type: DeliveryMethodTypeEnum
    url: str | None = Field(max_length=2048, default=None)

    model_config = ConfigDict(use_enum_values=True)



from pydantic import BaseModel, Field

from async_yookassa.enums.me_enums import MeStatusEnum, PayoutMethodsEnum
from async_yookassa.enums.payment_method_enums import PaymentMethodTypeEnum
from async_yookassa.models.me_submodels.fiscalization_model import Fiscalization
from async_yookassa.models.payment_submodels.amount_model import Amount


class Me(BaseModel):
    account_id: str
    status: MeStatusEnum
    test: bool
    fiscalization: Fiscalization | None = None
    fiscalization_enabled: bool | None = None
    payment_methods: list[PaymentMethodTypeEnum] | None = None
    itn: str | None = Field(min_length=1, max_length=20, default=None)
    payout_methods: list[PayoutMethodsEnum]
    name: str | None = None
    payout_balance: Amount | None = None



import platform
import sys
from typing import Self

import distro
from pydantic import BaseModel, Field

import async_yookassa
from async_yookassa.models.configuration_submodels.version_model import Version


class UserAgent(BaseModel):
    """
    Класс для создания заголовка User-Agent в запросах к API
    """

    os: Version = Field(default_factory=lambda: UserAgent.define_os())
    python: Version = Field(default_factory=lambda: UserAgent.define_python())
    sdk: Version = Field(default_factory=lambda: UserAgent.define_sdk())
    framework: Version | None = None
    cms: Version | None = None
    module: Version | None = None

    @staticmethod
    def define_os() -> Version:
        """Определение системы."""
        simple = UserAgent.__define_simple_os()
        if simple.name == "Windows":
            return Version(name=simple.name, version=simple.version)
        elif simple.name == "Linux":
            smart = UserAgent.__define_linux_os()
            return Version(name=smart.name.capitalize(), version=smart.version)
        return Version(name=simple.name, version=simple.version)

    @staticmethod
    def define_python() -> Version:
        """Определение версии Python."""
        info = sys.version_info
        version = f"{info.major}.{info.minor}.{info.micro}"
        return Version(name="Python", version=version)

    @staticmethod
    def define_sdk() -> Version:
        """Определение версии SDK."""
        version = async_yookassa.__version__
        return Version(name="Async YooKassa Python", version=version)

    def set_framework(self, name: str, version: str) -> Self:
        """Устанавливает версию фреймворка."""
        self.framework = Version(name=name, version=version)
        return self

    def set_cms(self, name: str, version: str) -> Self:
        """Устанавливает версию CMS."""
        self.cms = Version(name=name, version=version)
        return self

    def set_module(self, name: str, version: str) -> Self:
        """Устанавливает версию модуля."""
        self.module = Version(name=name, version=version)
        return self

    def get_header_string(self) -> str:
        """Возвращает значения header в виде строки."""
        part_delimiter = " "
        headers = [str(self.os), str(self.python)]
        if self.framework:
            headers.append(str(self.framework))
        if self.cms:
            headers.append(str(self.cms))
        if self.module:
            headers.append(str(self.module))
        headers.append(str(self.sdk))
        return part_delimiter.join(headers)

    @staticmethod
    def __define_simple_os() -> Version:
        """Определение данных системы для Windows."""
        return Version(name=platform.system(), version=platform.release())

    @staticmethod
    def __define_linux_os() -> Version:
        """Определение данных системы для Linux."""
        return Version(name=distro.name(), version=distro.version())



from enum import Enum


class MeStatusEnum(str, Enum):
    enabled = "enabled"
    disabled = "disabled"


class PayoutMethodsEnum(str, Enum):
    bank_card = "bank_card"
    yoo_money = "yoo_money"
    sbp = "sbp"



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount_model import Amount


class Settlement(BaseModel):
    type: str = Field(default="payout")
    amount: Amount



from pydantic import BaseModel, ConfigDict, model_validator

from async_yookassa.enums.vat_data_enums import RateEnum, VatDataTypeEnum
from async_yookassa.models.payment_submodels.amount_model import Amount


class VatData(BaseModel):
    type: VatDataTypeEnum
    amount: Amount | None = None
    rate: RateEnum | None = None

    model_config = ConfigDict(use_enum_values=True)

    @model_validator(mode="before")
    def validate_required_fields(cls, values):
        type_value = values.get("type")

        if type_value == VatDataTypeEnum.calculated:
            if not values.get("amount"):
                raise ValueError("Field 'payment_purpose' are required for type 'calculated'")
            if not values.get("rate"):
                raise ValueError("Field 'vat_data' are required for type 'calculated'")
        elif type_value == VatDataTypeEnum.mixed:
            if not values.get("amount"):
                raise ValueError("Field 'payment_purpose' are required for type 'mixed'")

        return values



from enum import Enum


class PaymentResponseStatusEnum(str, Enum):
    pending = "pending"
    waiting_for_capture = "waiting_for_capture"
    succeeded = "succeeded"
    canceled = "canceled"


class ReceiptRegistrationEnum(str, Enum):
    pending = "pending"
    succeeded = "succeeded"
    canceled = "canceled"



from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.payment_response_enums import (
    PaymentResponseStatusEnum,
    ReceiptRegistrationEnum,
)
from async_yookassa.models.payment_submodels.amount_model import Amount
from async_yookassa.models.payment_submodels.authorization_details_model import (
    AuthorizationDetails,
)
from async_yookassa.models.payment_submodels.cancellation_details_model import (
    CancellationDetails,
)
from async_yookassa.models.payment_submodels.confirmation_model import Confirmation
from async_yookassa.models.payment_submodels.deal_model import Deal
from async_yookassa.models.payment_submodels.invoice_details_model import InvoiceDetails
from async_yookassa.models.payment_submodels.payment_method_model import PaymentMethod
from async_yookassa.models.payment_submodels.recipient_model import RecipientResponse
from async_yookassa.models.payment_submodels.transfers_model import TransferResponse


class PaymentResponse(BaseModel):
    id: str = Field(min_length=36, max_length=36)
    status: PaymentResponseStatusEnum
    amount: Amount
    income_amount: Amount | None = None
    description: str | None = Field(max_length=128, default=None)
    recipient: RecipientResponse
    payment_method: PaymentMethod | None = None
    captured_at: datetime | None = None
    created_at: datetime
    expires_at: datetime | None = None
    confirmation: Confirmation | None = None
    test: bool
    refunded_amount: Amount | None = None
    paid: bool
    refundable: bool
    receipt_registration: ReceiptRegistrationEnum | None = None
    metadata: dict[str, Any] | None = None
    cancellation_details: CancellationDetails | None = None
    authorization_details: AuthorizationDetails | None = None
    transfers: list[TransferResponse] | None = None
    deal: Deal | None = None
    merchant_customer_id: str | None = Field(max_length=200, default=None)
    invoice_details: InvoiceDetails | None = None

    model_config = ConfigDict(use_enum_values=True)



from datetime import datetime

from pydantic import BaseModel, Field


class ReceiptOperationalDetails(BaseModel):
    operation_id: int = Field(gt=0, lt=255)
    value: str = Field(max_length=64)
    created_at: datetime



from pydantic import BaseModel

from async_yookassa.models.payment_response_model import PaymentResponse


class PaymentListResponse(BaseModel):
    type: str
    items: list[PaymentResponse]
    next_cursor: str | None = None



from enum import Enum


class FiscalizationProviderEnum(str, Enum):
    fns = "fns"
    avanpost = "avanpost"
    a_qsi = "a_qsi"
    atol = "atol"
    business_ru = "business_ru"
    evotor = "evotor"
    first_ofd = "first_ofd"
    kit_invest = "kit_invest"
    life_pay = "life_pay"
    mertrade = "mertrade"
    modul_kassa = "modul_kassa"
    rocket = "rocket"
    shtrih_m = "shtrih_m"



from pydantic import BaseModel, EmailStr, Field

from async_yookassa.models.payment_submodels.receipt_submodels.customer_model import (
    Customer,
)
from async_yookassa.models.payment_submodels.receipt_submodels.payment_subject_industry_details_model import (
    PaymentSubjectIndustryDetails,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_item_model import (
    ReceiptItem,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_operational_details_model import (
    ReceiptOperationalDetails,
)


class Receipt(BaseModel):
    customer: Customer | None = None
    items: list[ReceiptItem]
    phone: str | None = Field(min_length=11, max_length=11, default=None)
    email: EmailStr | None = None
    tax_system_code: int | None = Field(le=6, default=None)
    receipt_industry_details: list[PaymentSubjectIndustryDetails] | None = None
    receipt_operational_details: ReceiptOperationalDetails | None = None



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount_model import Amount


class ElectronicCertificate(BaseModel):
    amount: Amount
    basket_id: str


class Certificate(BaseModel):
    certificate_id: str = Field(min_length=20, max_length=30)
    tru_quantity: int
    available_compensation: Amount
    applied_compensation: Amount



from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.transfers_enums import TransferStatusEnum
from async_yookassa.models.payment_submodels.amount_model import Amount


class TransferBase(BaseModel):
    account_id: str
    amount: Amount
    description: str | None = Field(max_length=128, default=None)
    metadata: dict[str, Any] | None = None


class Transfer(TransferBase):
    platform_fee_amount: Amount


class TransferResponse(TransferBase):
    status: TransferStatusEnum
    platform_fee_amount: Amount | None = None

    model_config = ConfigDict(use_enum_values=True)



from datetime import datetime, timedelta
from decimal import Decimal
from typing import Any, Self

from pydantic import BaseModel, Field, model_validator

from async_yookassa.enums.payment_method_data_enums import PaymentMethodDataTypeEnum
from async_yookassa.models.payment_submodels.airline_model import Airline
from async_yookassa.models.payment_submodels.amount_model import Amount
from async_yookassa.models.payment_submodels.confirmation_model import Confirmation
from async_yookassa.models.payment_submodels.deal_model import Deal
from async_yookassa.models.payment_submodels.payment_method_data_model import (
    PaymentMethodData,
)
from async_yookassa.models.payment_submodels.receipt_model import Receipt
from async_yookassa.models.payment_submodels.receiver_model import Receiver
from async_yookassa.models.payment_submodels.recipient_model import Recipient
from async_yookassa.models.payment_submodels.transfers_model import Transfer


class PaymentData(BaseModel):
    amount: Amount
    receipt: Receipt | None = None
    recipient: Recipient | None = None
    save_payment_method: bool = False
    capture: bool = False
    client_ip: str | None = None
    description: str | None = Field(max_length=128, default=None)
    metadata: dict[str, Any] | None = None


class PaymentRequest(PaymentData):
    payment_token: str | None = None
    payment_method_id: str | None = None
    payment_method_data: PaymentMethodData | None = None
    confirmation: Confirmation | None = None
    airline: Airline | None = None
    transfers: list[Transfer] | None = None
    deal: Deal | None = None
    merchant_customer_id: str | None = Field(max_length=200, default=None)
    receiver: Receiver | None = None

    @model_validator(mode="after")
    def validate_data(cls, values: Self):
        amount = values.amount
        if amount is None or Decimal(amount.value) <= Decimal("0.0"):
            raise ValueError("Invalid or unspecified payment amount")

        receipt = values.receipt
        if receipt and receipt.has_items:
            if not (receipt.email or receipt.phone):
                raise ValueError("Both email and phone values are empty in receipt")
            if receipt.tax_system_code is None:
                for item in receipt.items:
                    if item.vat_code is None:
                        raise ValueError("Item vat_code and receipt tax_system_code not specified")

        payment_token = values.payment_token
        payment_method_id = values.payment_method_id
        payment_method_data = values.payment_method_data

        if payment_token:
            if payment_method_id:
                raise ValueError("Both payment_token and payment_method_id values are specified")
            if payment_method_data:
                raise ValueError("Both payment_token and payment_method_data values are specified")
        elif payment_method_id and payment_method_data:
            raise ValueError("Both payment_method_id and payment_method_data values are specified")

        if payment_method_data and payment_method_data.type == PaymentMethodDataTypeEnum.bank_card:
            card = payment_method_data.card
            if card:
                date_now = datetime.now() - timedelta(hours=27)
                date_expiry = (
                    datetime(year=int(card.expiry_year), month=int(card.expiry_month), day=1)
                    + timedelta(days=31)
                    - timedelta(days=1)
                )
                if date_now >= date_expiry:
                    raise ValueError("Card expired")

        return values
